@@ -17,32 +17,45 @@
 # You should have received a copy of the GNU General Public License
 # along with mb-netmgmt. If not, see <https://www.gnu.org/licenses/
 
-from lxml import etree
+import re
+from socketserver import BaseRequestHandler
+from socketserver import TCPServer as Server
+
 from ncclient.devices.default import DefaultDeviceHandler
 from ncclient.manager import connect
 from ncclient.transport.parser import DefaultXMLParser
-from ncclient.transport.session import SessionListener
-from ncclient.transport.ssh import (
-    END_DELIM,
-    MSG_DELIM,
-    PORT_NETCONF_DEFAULT,
-    SSHSession,
+from ncclient.transport.session import (
+    HelloHandler,
+    SessionListener,
+    qualify,
+    sub_ele,
+    to_ele,
+    to_xml,
 )
+from ncclient.transport.ssh import BUF_SIZE, MSG_DELIM, PORT_NETCONF_DEFAULT, SSHSession
 
-from mb_netmgmt.ssh import Handler as SshHandler
-from mb_netmgmt.ssh import Server, get_message_id, replace_message_id
+from mb_netmgmt.__main__ import Protocol
+from mb_netmgmt.ssh import accept
+
+stopped = False
+message_id_regex = ' message-id="([^"]*)"'
 
 
-class Handler(SshHandler):
-    message_terminators = [MSG_DELIM, END_DELIM]
-    default_port = 830
-
+class Handler(BaseRequestHandler, Protocol):
     def __init__(self, request, client_address, server):
         session = SSHSession(DefaultDeviceHandler())
         session.add_listener(Listener(super().handle_request))
         self.parser = DefaultXMLParser(session)
         super().__init__(request, client_address, server)
 
+    def handle(self):
+        self.callback_url = self.server.callback_url
+        self.channel = accept(self.request)
+        self.open_upstream()
+        self.handle_prompt()
+        while not stopped:
+            self.read_message()
+
     def open_upstream(self):
         to = self.get_to()
         if not to:
@@ -56,7 +69,16 @@ class Handler(SshHandler):
         )
 
     def handle_prompt(self):
-        pass
+        hello = to_ele(HelloHandler.build(["urn:ietf:params:netconf:base:1.0"], None))
+
+        # A server sending the <hello> element MUST include a <session-id>
+        # element containing the session ID for this NETCONF session.
+        # https://datatracker.ietf.org/doc/html/rfc6241#section-8.1
+        session_id = sub_ele(hello, "session-id")
+        session_id.text = "1"
+
+        self.channel.sendall(to_xml(hello) + MSG_DELIM.decode())
+        self.read_message()
 
     def read_proxy_response(self):
         return {
@@ -65,16 +87,17 @@ class Handler(SshHandler):
 
     def send_upstream(self, request, request_id):
         self.rpc_reply = self.manager.rpc(
-            etree.XML(request["command"][: -len(MSG_DELIM)])[0]
+            to_ele(request["command"][: -len(MSG_DELIM)])[0]
         )
 
-    def read_message(self, channel):
-        message = super().read_message(channel)
+    def read_message(self):
+        message = self.channel.recv(BUF_SIZE)
         self.parser.parse(message)
         return b""
 
-    def handle_request(self, request, request_id):
-        pass
+    def respond(self, response, request_id):
+        response = replace_message_id(response["response"], request_id)
+        self.channel.sendall(response)
 
 
 class Listener(SessionListener):
@@ -82,6 +105,20 @@ class Listener(SessionListener):
         self.handle_request = handle_request
 
     def callback(self, root, raw):
+        tag, attrs = root
+        if (tag == qualify("hello")) or (tag == "hello"):
+            return
         request = {"command": replace_message_id(raw, "") + MSG_DELIM.decode()}
         request_id = get_message_id(raw)
         self.handle_request(request, request_id)
+
+
+def get_message_id(rpc):
+    try:
+        return re.findall(message_id_regex, rpc)[0]
+    except IndexError:
+        return None
+
+
+def replace_message_id(rpc, message_id):
+    return re.sub(message_id_regex, f' message-id="{message_id}"', rpc)
