@@ -80,10 +80,12 @@ def test_create_netconf_server():
     with ncclient.manager.connect(
         host="localhost", port=port, password="", hostkey_verify=False
     ) as m:
+        assert m.session_id == "1"
+        assert "urn:ietf:params:netconf:base:1.0" in m.server_capabilities
         m.get_config("running")
-    ssh.stopped = True
+    netconf.stopped = True
     server.shutdown()
-    ssh.stopped = False
+    netconf.stopped = False
     netconf.Handler.open_upstream = original_open_upstream
     netconf.Handler.post_request = original_post_request
 
